.row-added{background-color:#ccffd8}.row-added:hover,.row-added[aria-selected=true]{background-color:#c1f6cd}.row-added[aria-selected=true]:hover{background-color:#a6f2b8}.row-removed{background-color:#ffd7d5}.row-removed:hover,.row-removed[aria-selected=true]{background-color:#f6c3c1}.row-removed[aria-selected=true]:hover{background-color:#f2aaa6}.row-normal{background-color:#fff}.row-normal:hover,.row-normal[aria-selected=true]{background-color:#f1f1f1}.row-normal[aria-selected=true]:hover{background-color:#e6e6e6}.row-selectable{cursor:pointer}.schema-column[aria-selected=true]{outline:none;outline-offset:0}.schema-column-index{text-align:right;filter:brightness(.95)}.column-body-type-changed{background-color:#ffd7d5}.row-normal:hover .column-body-type-changed,.row-normal[aria-selected=true] .column-body-type-changed{background-color:#f6c3c1}.row-normal[aria-selected=true]:hover .column-body-type-changed{background-color:#f2aaa6}.column-index-reordered{background-color:#ffad15}.row-normal:hover .column-index-reordered,.row-normal[aria-selected=true] .column-index-reordered{background-color:#f9a406}.row-normal[aria-selected=true]:hover .column-index-reordered{background-color:#e09306}